using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;

namespace ConvexHulls
{
    public partial class FormApp : Form
    {
        List<Point> points = new List<Point>();
        Point[] hull = null;
        Graphics g = null;
        int border = 20;
        
        public FormApp()
        {
            InitializeComponent();
            g = panel.CreateGraphics();

            /*int n = 100;
            for (int i = 0; i < n; i++)
            {
                double p = 2 * i * Math.PI / n;
                points.Add(new Point((int)(Math.Cos(p) * 200 + 210), (int)(Math.Sin(p) * 200 + 210)));
            }*/

            /*
            points.Add(new Point(25, 25));
            points.Add(new Point(25, 50));
            points.Add(new Point(25, 75));
            points.Add(new Point(25, 100));
            points.Add(new Point(25, 125));

            draw();
            */
        }

        private void buttonClear_Click(object sender, EventArgs e)
        {
            g.Clear(Color.White);
            points.Clear();
            hull = null;
            panel.Update();
        }

        private void buttonClearHull_Click(object sender, EventArgs e)
        {
            g.Clear(Color.White);
            hull = null;
            draw();
        }

        private void buttonCreate_Click(object sender, EventArgs e)
        {
            g.Clear(Color.White);
            hull = null;
            Random rnd = new Random();
            points.Clear();
            for (int i = 0; i < numericPoints.Value; i++)
                points.Add(new Point(
                    rnd.Next(border - 1, panel.Width - 3 - border), 
                    rnd.Next(border - 1, panel.Height - 3 - border)
                ));
            draw();
        }

        private void buttonCalc_Click(object sender, EventArgs e)
        {
            if (points.Count < 1)
                return;

            HiResTimer timer = new HiResTimer();
            IConvexHull hullAlg = null;
            string hullAlgName = "";

            if (radioBruteForce.Checked)
            {
                hullAlgName = radioBruteForce.Text;
                hullAlg = new BruteForceHull();
            }
            else if (radioGraham.Checked)
            {
                hullAlgName = radioGraham.Text;
                hullAlg = new GrahamScan();
            }
            else if (radioJarvis.Checked)
            {
                hullAlgName = radioJarvis.Text;
                hullAlg = new JarvisMarch();
            }
            else if (radioQuickhull.Checked)
            {
                hullAlgName = radioQuickhull.Text;
                hullAlg = new QuickHull();
            }
            else if (radioMerge.Checked)
            {
                hullAlgName = radioMerge.Text;
                hullAlg = new MergeHull();
            }
            else if (radioChan.Checked)
            {
                hullAlgName = radioChan.Text;
                hullAlg = new ChanHull();
            }

            hull = null;
            g.Clear(Color.White);
            labelAlg.Text = hullAlgName + ":";
            if (hullAlg != null)
            {
                hull = null;
                long st = timer.Value;
                List<Point> plist = points;
                if (checkAkl.Checked)
                {
                    Point[] pa = AklToussaintHeuristic.PrunePoints(points.ToArray());
                    this.hull = hullAlg.CalculateHull(pa);
                }
                else
                    this.hull = hullAlg.CalculateHull(plist.ToArray());
                long et = timer.Value;
                labelStatus.Text = "Ellapsed time: " + ((et - st) / (float)timer.Frequency + "s");
            }
            else
            {
                labelStatus.Text = "Not implemented yet...";
            }
            draw();
        }

        private void draw()
        {
            if (hull != null)
            {
                Point[] hull2 = hull;
                // workaround for unordered brute force hull
                if (radioBruteForce.Checked)
                {
                    QuickHull qhull = new QuickHull();
                    hull2 = qhull.CalculateHull(points.ToArray());
                }
                g.FillPolygon(Brushes.GhostWhite, hull2);
                if(hull2.Length > 1)
                    g.DrawPolygon(Pens.Blue, hull2);
            }
            Pen pen = new Pen(Color.Black, 2);
            foreach (Point p in points)
                g.DrawRectangle(pen, p.X, p.Y, 1, 1);
            Pen penRed = new Pen(Color.Red, 2);
            if (hull != null)
            {
                foreach (Point p in hull)
                    g.DrawRectangle(penRed, p.X, p.Y, 1, 1);
            }
            panel.Update();
        }

        private void panel_MouseClick(object sender, MouseEventArgs e)
        {
            Point p = new Point(e.X, e.Y);
            points.Add(p);
            if (hull != null)
            {
                g.Clear(Color.White);
                hull = null;
                draw();
            }
            else
            {
                Pen pen = new Pen(Color.Black, 2);
                g.DrawRectangle(pen, p.X, p.Y, 1, 1);
            }
        }

        private void panel_Paint(object sender, PaintEventArgs e)
        {
            draw();
        }

        private void buttonStatExp_Click(object sender, EventArgs e)
        {
            StatisticExperiments exp = new StatisticExperiments();
            exp.Run();
        }
    }
}