package algorithms.priorityqueues;

public class LeftistTreeNode<K,V> implements PriorityQueueNode<K,V> {

	int priority = -1;
	int dist = 0;
	V value = null;
	K key = null;
	
	LeftistTreeNode<K,V> parent = null;
	LeftistTreeNode<K,V> left = null;
	LeftistTreeNode<K,V> right = null;
	
	
	public LeftistTreeNode(V val, K key, int priority){
		this.value = val;
		this.key = key;
		this.priority = priority;
	}
	public LeftistTreeNode(V val, K key, int priority, int dist){
		this(val, key, priority);
		this.dist = dist;
	}
	public LeftistTreeNode(V val, K key, int priority, int dist, LeftistTreeNode<K,V> parent, LeftistTreeNode<K,V> left, LeftistTreeNode<K,V> right){
		this(val, key, priority, dist);
		this.parent = parent;
		this.left = left;
		this.right = right;
	}
	
	public int getPriority() {
		return priority;
	}

	public void setPriority(int val) {
		this.priority = val;
	}
	
	public K getKey(){
		return key;
	}
	
	public void setKey(K key){
		this.key = key;
	}
	
	public V getValue() {
		return value;
	}
	public void setValue(V val) {
		this.value = val;
	}
	
	public int getDistance(){
		return dist;
	}
	
	public void setDistance(int dist){
		this.dist = dist;
	}
	
	public LeftistTreeNode<K,V> getParent(){
		return parent;
	}
	
	public void setParent(LeftistTreeNode<K,V> node){
		this.parent = node;
	}
	
	public LeftistTreeNode<K,V> getLeftChild(){
		return left;
	}
	
	public void setLeftChild(LeftistTreeNode<K,V> node){
		this.left = node;
	}
	
	public LeftistTreeNode<K,V> getRightChild(){
		return right;
	}
	
	public void setRightChild(LeftistTreeNode<K,V> node){
		this.right = node;
	}
	
	
	public void updateDistance(){
		if(right == null)
			setDistance(0);
		else
			setDistance(1 + right.getDistance());
	}
	
	public void swapChildren(){
		LeftistTreeNode<K,V> tmp = left;
		left = right;
		right = tmp;
	}
	
	public LeftistTreeNode<K,V> getSibling(){
		if(parent != null){
			if(parent.left == this)
				return parent.right;
			else
				return parent.left;
		}
		
		return null;
	}
	
	public boolean isLeftChild(){
		if(parent != null)
			return (parent.left == this);
		return false;
	}
	
	public boolean isRightChild(){
		return !isLeftChild();
	}
	
}
