package mains;

import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.lang.management.ManagementFactory;
import java.lang.management.ThreadMXBean;
import java.util.Random;

import algorithms.priorityqueues.*;

public class Excercise2 {
	private static int runs = 100;      // number of runs to build the mean over
	private static int max = 300000;    // maximum of elements in one prio queue
	private static int div = 10000;     // difference of elements between two test cases
	
	// init of prio queues combined with tests of element inserts
	private static void initInsertTests() throws IOException{
		int start = div;
		int is = max/10;   // insert size
		ThreadMXBean timer = ManagementFactory.getThreadMXBean();
		timer.setThreadCpuTimeEnabled(true);
		
		System.out.println("\n ==> initInsertTests()...");
		Random rnd = new Random();
		double[][] initTime = new double[2][(max-start+div)/div];
		double[][] insertTime = new double[2][(max-start+div)/div];
		for(int n=start, k=0; n<=max; n+=div, k++){
			System.out.println("\nElemente: " + n);
			for(int i=1; i<=runs; i++){
				System.out.print(i+",");
				Integer[] keys = new Integer[n];
				int[] prios = new int[n];
				Integer[] insertKeys = new Integer[is];
				int[] insertPrios = new int[is];
				for(int j=0; j<keys.length; j++){
					keys[j] = j;
					prios[j] = rnd.nextInt(n*10) + 1;
				}
				for(int j=0; j<insertKeys.length; j++){
					insertKeys[j] = keys.length + j;
					insertPrios[j] = rnd.nextInt(n*10) + 1;
				}
				
				long ts = timer.getCurrentThreadCpuTime();
				LeftistTree<Integer,Integer> ltree = new LeftistTree<Integer,Integer>(keys,keys,prios);
				long te = timer.getCurrentThreadCpuTime();
				initTime[0][k] += (te-ts)*Math.pow(10,-9);
				
				ts = timer.getCurrentThreadCpuTime();
				for(int j=0; j<insertKeys.length; j++){
					ltree.insert(new LeftistTreeNode<Integer,Integer>(insertKeys[j],insertKeys[j],insertPrios[j]));
				}
				te = timer.getCurrentThreadCpuTime();
				insertTime[0][k] += (te-ts)*Math.pow(10,-9);
				
				ts = timer.getCurrentThreadCpuTime();
				BinomialHeap<Integer,Integer> binheap = new BinomialHeap<Integer,Integer>(keys,keys,prios);
				te = timer.getCurrentThreadCpuTime();
				initTime[1][k] += (te-ts)*Math.pow(10,-9);
				
				ts = timer.getCurrentThreadCpuTime();
				for(int j=0; j<insertKeys.length; j++){
					binheap.insert(new BinomialHeapNode<Integer,Integer>(insertKeys[j],insertKeys[j],insertPrios[j]));
				}
				te = timer.getCurrentThreadCpuTime();
				insertTime[1][k] += (te-ts)*Math.pow(10,-9);
			}
			System.out.println();
			initTime[0][k] /= runs;
			initTime[1][k] /= runs;
			insertTime[0][k] /= runs;
			insertTime[1][k] /= runs;
			System.out.println("LeftistTree: initTime="+initTime[0][k]+"s, insertTime="+insertTime[0][k]+"s");
			System.out.println("BinHeap: initTime="+initTime[1][k]+"s, insertTime="+insertTime[1][k]+"s");
		}
		
		// write the results into a file
		Writer osinit = new FileWriter("InitCmp.txt");
		Writer osinsert = new FileWriter("InsertCmp.txt");
		for(int i=0; i<2; i++){
			switch(i){
			case 0:
				osinit.write("# Leftist Tree\n");
				osinsert.write("# Leftist Tree\n");
				break;
			case 1:
				osinit.write("# Binomial Heap\n");
				osinsert.write("# Binomial Heap\n");
				break;
			}
			for(int j=0; j<initTime[0].length; j++){
				osinit.write((start+j*div) + " " + initTime[i][j] + "\n");
				osinsert.write((start+j*div) + " " + insertTime[i][j] + "\n");
			}
			osinit.write("\n\n");
			osinsert.write("\n\n");
		}
		osinit.close();
		osinsert.close();
	}
	
	// get minimum combined with tests for delete minimum
	private static void getMinDelMinTests() throws IOException{
		int start = max/10;
		int ds = max/10;   // delete size
		ThreadMXBean timer = ManagementFactory.getThreadMXBean();
		timer.setThreadCpuTimeEnabled(true);
		
		System.out.println("\n ==> getMinDelMinTests()...");
		Random rnd = new Random();
		double[][] getminTime = new double[2][(max-start+div)/div];
		double[][] delminTime = new double[2][(max-start+div)/div];
		for(int n=start, k=0; n<=max; n+=div, k++){
			System.out.println("Elemente: " + n);
			for(int i=1; i<=runs; i++){
				System.out.print(i+",");
				Integer[] keys = new Integer[n];
				int[] prios = new int[n];
				for(int j=0; j<keys.length; j++){
					keys[j] = j;
					prios[j] = rnd.nextInt(n*10) + 1;
				}
				
				LeftistTree<Integer,Integer> ltree = new LeftistTree<Integer,Integer>(keys,keys,prios);
				long ts = timer.getCurrentThreadCpuTime();
				for(int j=0; j<ds; j++)
					ltree.getMinimum();
				long te = timer.getCurrentThreadCpuTime();
				getminTime[0][k] += (te-ts)*Math.pow(10,-9);
				
				ts = timer.getCurrentThreadCpuTime();
				for(int j=0; j<ds; j++)
					ltree.deleteMinimum();
				te = timer.getCurrentThreadCpuTime();
				delminTime[0][k] += (te-ts)*Math.pow(10,-9);
				
				BinomialHeap<Integer,Integer> binheap = new BinomialHeap<Integer,Integer>(keys,keys,prios);
				ts = timer.getCurrentThreadCpuTime();
				for(int j=0; j<ds; j++)
					binheap.getMinimum();
				te = timer.getCurrentThreadCpuTime();
				getminTime[1][k] += (te-ts)*Math.pow(10,-9);
				
				ts = timer.getCurrentThreadCpuTime();
				for(int j=0; j<ds; j++)
					binheap.deleteMinimum();
				te = timer.getCurrentThreadCpuTime();
				delminTime[1][k] += (te-ts)*Math.pow(10,-9);
			}
			System.out.println();
			getminTime[0][k] /= runs;
			getminTime[1][k] /= runs;
			delminTime[0][k] /= runs;
			delminTime[1][k] /= runs;
			System.out.println("LeftistTree: getminTime="+getminTime[0][k]+"s, delminTime="+delminTime[0][k]+"s");
			System.out.println("BinHeap: getminTime="+getminTime[1][k]+"s, delminTime="+delminTime[1][k]+"s");
		}
		
		// write the results into a file
		Writer osgetmin = new FileWriter("GetMinCmp.txt");
		Writer osdelmin = new FileWriter("DelMinCmp.txt");
		for(int i=0; i<2; i++){
			switch(i){
			case 0:
				osgetmin.write("# Leftist Tree\n");
				osdelmin.write("# Leftist Tree\n");
				break;
			case 1:
				osgetmin.write("# Binomial Heap\n");
				osdelmin.write("# Binomial Heap\n");
				break;
			}
			for(int j=0; j<getminTime[0].length; j++){
				osgetmin.write((start+j*div) + " " + getminTime[i][j] + "\n");
				osdelmin.write((start+j*div) + " " + delminTime[i][j] + "\n");
			}
			osgetmin.write("\n\n");
			osdelmin.write("\n\n");
		}
		osgetmin.close();
		osdelmin.close();
	}
	
	// deletion of elements from a priority queue
	private static void deleteTests() throws IOException{
		int start = max/10;
		int ds = max/10;   // delete size
		ThreadMXBean timer = ManagementFactory.getThreadMXBean();
		timer.setThreadCpuTimeEnabled(true);
		
		System.out.println("\n ==> deleteTests()...");
		Random rnd = new Random();
		double[][] deleteTime = new double[2][(max-start+div)/div];
		for(int n=start, k=0; n<=max; n+=div, k++){
			System.out.println("Elemente: " + n);
			for(int i=1; i<=runs; i++){
				System.out.print(i+",");
				Integer[] keys = new Integer[n];
				int[] prios = new int[n];
				for(int j=0; j<keys.length; j++){
					keys[j] = j;
					prios[j] = rnd.nextInt(n*10) + 1;
				}
				
				LeftistTree<Integer,Integer> ltree = new LeftistTree<Integer,Integer>(keys,keys,prios);
				long ts = timer.getCurrentThreadCpuTime();
				for(int j=0; j<ds; j++){
					ltree.delete(ltree.getNode(j));
				}
				long te = timer.getCurrentThreadCpuTime();
				deleteTime[0][k] += (te-ts)*Math.pow(10,-9);
				
				BinomialHeap<Integer,Integer> binheap = new BinomialHeap<Integer,Integer>(keys,keys,prios);
				ts = timer.getCurrentThreadCpuTime();
				for(int j=0; j<ds; j++){
					binheap.delete(binheap.getNode(j));
				}
				te = timer.getCurrentThreadCpuTime();
				deleteTime[1][k] += (te-ts)*Math.pow(10,-9);
			}
			System.out.println();
			deleteTime[0][k] /= runs;
			deleteTime[1][k] /= runs;
			System.out.println("LeftistTree: deleteTime="+deleteTime[0][k]+"s");
			System.out.println("BinHeap: deleteTime="+deleteTime[1][k]+"s");
		}
		
		Writer osdelete = new FileWriter("DeleteCmp.txt");
		for(int i=0; i<2; i++){
			switch(i){
			case 0:
				osdelete.write("# Leftist Tree\n");
				break;
			case 1:
				osdelete.write("# Binomial Heap\n");
				break;
			}
			for(int j=0; j<deleteTime[0].length; j++){
				osdelete.write((start+j*div) + " " + deleteTime[i][j] + "\n");
			}
			osdelete.write("\n\n");
		}
		osdelete.close();
	}
	
	// decrease priorities of elements from a priority queue
	private static void decreasePrioTests() throws IOException{
		int start = max/10;
		int ds = max/10;   // decrease priorities size
		ThreadMXBean timer = ManagementFactory.getThreadMXBean();
		timer.setThreadCpuTimeEnabled(true);
		
		System.out.println("\n ==> decreasePrioTests()...");
		Random rnd = new Random();
		double[][] decPrioTime = new double[2][(max-start+div)/div];
		for(int n=start, k=0; n<=max; n+=div, k++){
			System.out.println("Elemente: " + n);
			for(int i=1; i<=runs; i++){
				System.out.print(i+",");
				Integer[] keys = new Integer[n];
				int[] prios = new int[n];
				for(int j=0; j<keys.length; j++){
					keys[j] = j;
					prios[j] = rnd.nextInt(n*10) + 1;
				}
				int[] newPrios = new int[ds];
				for(int j=0; j<newPrios.length; j++)
					newPrios[j] = (int)(prios[j]*(rnd.nextDouble()+0.1)*0.8);
				
				LeftistTree<Integer,Integer> ltree = new LeftistTree<Integer,Integer>(keys,keys,prios);
				long ts = timer.getCurrentThreadCpuTime();
				for(int j=0; j<ds; j++){
					ltree.decreasePriority(ltree.getNode(j), newPrios[j]);
				}
				long te = timer.getCurrentThreadCpuTime();
				decPrioTime[0][k] += (te-ts)*Math.pow(10,-9);
				
				BinomialHeap<Integer,Integer> binheap = new BinomialHeap<Integer,Integer>(keys,keys,prios);
				ts = timer.getCurrentThreadCpuTime();
				for(int j=0; j<ds; j++){
					binheap.decreasePriority(binheap.getNode(j), newPrios[j]);
				}
				te = timer.getCurrentThreadCpuTime();
				decPrioTime[1][k] += (te-ts)*Math.pow(10,-9);
			}
			System.out.println();
			decPrioTime[0][k] /= runs;
			decPrioTime[1][k] /= runs;
			System.out.println("LeftistTree: decPrioTime="+decPrioTime[0][k]+"s");
			System.out.println("BinHeap: decPrioTime="+decPrioTime[1][k]+"s");
		}
		
		Writer osdecprio = new FileWriter("DecreasePrioCmp.txt");
		for(int i=0; i<2; i++){
			switch(i){
			case 0:
				osdecprio.write("# Leftist Tree\n");
				break;
			case 1:
				osdecprio.write("# Binomial Heap\n");
				break;
			}
			for(int j=0; j<decPrioTime[0].length; j++){
				osdecprio.write((start+j*div) + " " + decPrioTime[i][j] + "\n");
			}
			osdecprio.write("\n\n");
		}
		osdecprio.close();
	}
	
	// merge two priority queues
	private static void mergeTests() throws IOException{
		int max = 250000;
		ThreadMXBean timer = ManagementFactory.getThreadMXBean();
		timer.setThreadCpuTimeEnabled(true);
		
		System.out.println("\n ==> mergeTests()...");
		Random rnd = new Random();
		double[][] mergeTime = new double[2][max/div];
		for(int n=div, k=0; n<=max; n+=div, k++){
			System.out.println("Elemente: " + n);
			for(int i=1; i<=runs; i++){
				System.out.print(i+",");
				int n1 = rnd.nextInt(n-1)+1;
				int n2 = n-n1;
				Integer[] keys1 = new Integer[n1];
				Integer[] keys2 = new Integer[n2];
				int[] prios1 = new int[n1];
				int[] prios2 = new int[n2];
				for(int j=0; j<keys1.length; j++){
					keys1[j] = j;
					prios1[j] = rnd.nextInt(n*10) + 1;
				}
				for(int j=0; j<keys2.length; j++){
					keys2[j] = keys1.length + j;
					prios2[j] = rnd.nextInt(n*10) + 1;
				}
				
				LeftistTree<Integer,Integer> ltree1 = new LeftistTree<Integer,Integer>(keys1,keys1,prios1);
				LeftistTree<Integer,Integer> ltree2 = new LeftistTree<Integer,Integer>(keys2,keys2,prios2);
				long ts = timer.getCurrentThreadCpuTime();
				ltree1.merge(ltree2);
				long te = timer.getCurrentThreadCpuTime();
				mergeTime[0][k] += (te-ts)*Math.pow(10,-9);
				
				BinomialHeap<Integer,Integer> binheap1 = new BinomialHeap<Integer,Integer>(keys1,keys1,prios1);
				BinomialHeap<Integer,Integer> binheap2 = new BinomialHeap<Integer,Integer>(keys2,keys2,prios2);
				ts = timer.getCurrentThreadCpuTime();
				binheap1.merge(binheap2);
				te = timer.getCurrentThreadCpuTime();
				mergeTime[1][k] += (te-ts)*Math.pow(10,-9);
			}
			System.out.println();
			mergeTime[0][k] /= runs;
			mergeTime[1][k] /= runs;
			System.out.println("LeftistTree: mergeTime="+mergeTime[0][k]+"s");
			System.out.println("BinHeap: mergeTime="+mergeTime[1][k]+"s");
		}
		
		Writer osmerge = new FileWriter("MergeCmp.txt");
		for(int i=0; i<2; i++){
			switch(i){
			case 0:
				osmerge.write("# Leftist Tree\n");
				break;
			case 1:
				osmerge.write("# Binomial Heap\n");
				break;
			}
			for(int j=0; j<mergeTime[0].length; j++){
				osmerge.write((div+j*div) + " " + mergeTime[i][j] + "\n");
			}
			osmerge.write("\n\n");
		}
		osmerge.close();
	}
	
	// leftist tree advantage
	private static void scenario1() throws IOException{
		int max = 400000;
		
		ThreadMXBean timer = ManagementFactory.getThreadMXBean();
		timer.setThreadCpuTimeEnabled(true);
		
		System.out.println("\n ==> scenario1()...");
		Random rnd = new Random();
		double[][] scenTime = new double[2][max/div];
		for(int n=div, k=0; n<=max; n+=div, k++){
			int sinit = (int)(n*0.5);
			int sdelmin = (int)(n*0.25);
			int sdelete = (int)(n*0.25);
			System.out.println("Elemente: " + n);
			for(int i=1; i<=runs; i++){
				System.out.print(i+",");
				Integer[] keys = new Integer[sinit];
				int[] prios = new int[sinit];
				for(int j=0; j<keys.length; j++){
					keys[j] = j;
					prios[j] = rnd.nextInt(n*10) + 1;
				}

				long ts = timer.getCurrentThreadCpuTime();
				// init
				LeftistTree<Integer,Integer> ltree = new LeftistTree<Integer,Integer>(keys,keys,prios);
				// delete
				for(int j=0; j<sdelete; j++){
					ltree.delete(ltree.getNode(j));
				}
				// delete minimum
				for(int j=0; j<sdelmin; j++){
					ltree.deleteMinimum();
				}
				long te = timer.getCurrentThreadCpuTime();
				scenTime[0][k] += (te-ts)*Math.pow(10,-9);
				
				ts = timer.getCurrentThreadCpuTime();
				// init
				BinomialHeap<Integer,Integer> binheap = new BinomialHeap<Integer,Integer>(keys,keys,prios);
				// delete
				for(int j=0; j<sdelete; j++){
					binheap.delete(binheap.getNode(j));
				}
				// delete minimum
				for(int j=0; j<sdelmin; j++){
					binheap.deleteMinimum();
				}
				te = timer.getCurrentThreadCpuTime();
				scenTime[1][k] += (te-ts)*Math.pow(10,-9);
			}
			System.out.println();
			scenTime[0][k] /= runs;
			scenTime[1][k] /= runs;
			System.out.println("LeftistTree: scenario1Time="+scenTime[0][k]+"s");
			System.out.println("BinHeap: scenario1Time="+scenTime[1][k]+"s");
		}
		
		Writer osscen1 = new FileWriter("Scenario1Cmp.txt");
		for(int i=0; i<2; i++){
			switch(i){
			case 0:
				osscen1.write("# Leftist Tree\n");
				break;
			case 1:
				osscen1.write("# Binomial Heap\n");
				break;
			}
			for(int j=0; j<scenTime[0].length; j++){
				osscen1.write((div+j*div) + " " + scenTime[i][j] + "\n");
			}
			osscen1.write("\n\n");
		}
		osscen1.close();
	}
	
	// binomial heap advantage
	private static void scenario2() throws IOException{
		int max = 400000;
		
		ThreadMXBean timer = ManagementFactory.getThreadMXBean();
		timer.setThreadCpuTimeEnabled(true);
		
		System.out.println("\n ==> scenario2()...");
		Random rnd = new Random();
		double[][] scenTime = new double[2][max/div];
		for(int n=div, k=0; n<=max; n+=div, k++){
			int sinit = (int)(n*0.5);
			int sdecprio = (int)(n*0.25);
			int sinsert = (int)(n*0.25);
			System.out.println("Elemente: " + n);
			for(int i=1; i<=runs; i++){
				System.out.print(i+",");
				Integer[] keys = new Integer[sinit];
				int[] prios = new int[sinit];
				for(int j=0; j<keys.length; j++){
					keys[j] = j;
					prios[j] = rnd.nextInt(n*10) + 1;
				}
				int[] newPrios = new int[sdecprio];
				for(int j=0; j<newPrios.length; j++)
					newPrios[j] = (int)(prios[j]*(rnd.nextDouble()+0.1)*0.8);
				Integer[] insertKeys = new Integer[sinsert];
				int[] insertPrios = new int[sinsert];
				for(int j=0; j<insertKeys.length; j++){
					insertKeys[j] = keys.length + j;
					insertPrios[j] = rnd.nextInt(n*10) + 1;
				}
				
				long ts = timer.getCurrentThreadCpuTime();
				// init
				LeftistTree<Integer,Integer> ltree = new LeftistTree<Integer,Integer>(keys,keys,prios);
				// decrease priority
				for(int j=0; j<sdecprio; j++){
					ltree.decreasePriority(ltree.getNode(j), newPrios[j]);
				}
				// insert
				for(int j=0; j<insertKeys.length; j++){
					ltree.insert(new LeftistTreeNode<Integer,Integer>(insertKeys[j],insertKeys[j],insertPrios[j]));
				}
				long te = timer.getCurrentThreadCpuTime();
				scenTime[0][k] += (te-ts)*Math.pow(10,-9);
				
				ts = timer.getCurrentThreadCpuTime();
				// init
				BinomialHeap<Integer,Integer> binheap = new BinomialHeap<Integer,Integer>(keys,keys,prios);
				// decrease priority
				for(int j=0; j<sdecprio; j++){
					binheap.decreasePriority(binheap.getNode(j), newPrios[j]);
				}
				// insert
				for(int j=0; j<insertKeys.length; j++){
					binheap.insert(new BinomialHeapNode<Integer,Integer>(insertKeys[j],insertKeys[j],insertPrios[j]));
				}
				te = timer.getCurrentThreadCpuTime();
				scenTime[1][k] += (te-ts)*Math.pow(10,-9);
			}
			System.out.println();
			scenTime[0][k] /= runs;
			scenTime[1][k] /= runs;
			System.out.println("LeftistTree: scenario2Time="+scenTime[0][k]+"s");
			System.out.println("BinHeap: scenario2Time="+scenTime[1][k]+"s");
		}
		
		Writer osscen2 = new FileWriter("Scenario2Cmp.txt");
		for(int i=0; i<2; i++){
			switch(i){
			case 0:
				osscen2.write("# Leftist Tree\n");
				break;
			case 1:
				osscen2.write("# Binomial Heap\n");
				break;
			}
			for(int j=0; j<scenTime[0].length; j++){
				osscen2.write((div+j*div) + " " + scenTime[i][j] + "\n");
			}
			osscen2.write("\n\n");
		}
		osscen2.close();
	}
	
	// leftist tree advantage
	private static void scenario31() throws IOException{
		int max = 600000;
		
		ThreadMXBean timer = ManagementFactory.getThreadMXBean();
		timer.setThreadCpuTimeEnabled(true);
		
		System.out.println("\n ==> scenario3()...");
		Random rnd = new Random();
		double[][] scenTime = new double[2][max/div];
		for(int n=div, k=0; n<=max; n+=div, k++){
			int sinit = (int)(n*0.4);
			int sdecprio = (int)(n*0.25);
			int sinsert = (int)(n*0.10);
			int sdelete = (int)(n*0.10);
			int sdelmin = (int)(n*0.15);
			System.out.println("Elemente: " + n);
			for(int i=1; i<=runs; i++){
				System.out.print(i+",");
				Integer[] keys = new Integer[sinit];
				int[] prios = new int[sinit];
				for(int j=0; j<keys.length; j++){
					keys[j] = j;
					prios[j] = rnd.nextInt(n*10) + 1;
				}
				int[] newPrios = new int[sdecprio];
				for(int j=0; j<newPrios.length; j++)
					newPrios[j] = (int)(prios[j]*(rnd.nextDouble()+0.1)*0.8);
				Integer[] insertKeys = new Integer[sinsert];
				int[] insertPrios = new int[sinsert];
				for(int j=0; j<insertKeys.length; j++){
					insertKeys[j] = keys.length + j;
					insertPrios[j] = rnd.nextInt(n*10) + 1;
				}
				
				long ts = timer.getCurrentThreadCpuTime();
				// init
				LeftistTree<Integer,Integer> ltree = new LeftistTree<Integer,Integer>(keys,keys,prios);
				// delete
				for(int j=0; j<sdelete; j++){
					ltree.delete(ltree.getNode(j));
				}
				// insert
				for(int j=0; j<insertKeys.length; j++){
					ltree.insert(new LeftistTreeNode<Integer,Integer>(insertKeys[j],insertKeys[j],insertPrios[j]));
				}
				// decrease priority
				for(int j=0; j<sdecprio; j++){
					ltree.decreasePriority(ltree.getNode(j), newPrios[j]);
				}
				// delete minimum
				for(int j=0; j<sdelmin; j++){
					ltree.deleteMinimum();
				}
				long te = timer.getCurrentThreadCpuTime();
				scenTime[0][k] += (te-ts)*Math.pow(10,-9);
				
				ts = timer.getCurrentThreadCpuTime();
				// init
				BinomialHeap<Integer,Integer> binheap = new BinomialHeap<Integer,Integer>(keys,keys,prios);
				// delete
				for(int j=0; j<sdelete; j++){
					binheap.delete(binheap.getNode(j));
				}
				// insert
				for(int j=0; j<insertKeys.length; j++){
					binheap.insert(new BinomialHeapNode<Integer,Integer>(insertKeys[j],insertKeys[j],insertPrios[j]));
				}
				// decrease priority
				for(int j=0; j<sdecprio; j++){
					binheap.decreasePriority(binheap.getNode(j), newPrios[j]);
				}
				// delete minimum
				for(int j=0; j<sdelmin; j++){
					binheap.deleteMinimum();
				}
				te = timer.getCurrentThreadCpuTime();
				scenTime[1][k] += (te-ts)*Math.pow(10,-9);
			}
			System.out.println();
			scenTime[0][k] /= runs;
			scenTime[1][k] /= runs;
			System.out.println("LeftistTree: scenario31Time="+scenTime[0][k]+"s");
			System.out.println("BinHeap: scenario31Time="+scenTime[1][k]+"s");
		}
		
		Writer osscen31 = new FileWriter("Scenario31Cmp.txt");
		for(int i=0; i<2; i++){
			switch(i){
			case 0:
				osscen31.write("# Leftist Tree\n");
				break;
			case 1:
				osscen31.write("# Binomial Heap\n");
				break;
			}
			for(int j=0; j<scenTime[0].length; j++){
				osscen31.write((div+j*div) + " " + scenTime[i][j] + "\n");
			}
			osscen31.write("\n\n");
		}
		osscen31.close();
	}
	
	// binomial heap advantage
	private static void scenario32() throws IOException{
		int max = 600000;
		
		ThreadMXBean timer = ManagementFactory.getThreadMXBean();
		timer.setThreadCpuTimeEnabled(true);
		
		System.out.println("\n ==> scenario4()...");
		Random rnd = new Random();
		double[][] scenTime = new double[2][max/div];
		for(int n=div, k=0; n<=max; n+=div, k++){
			int sinit = (int)(n*0.45);
			int sdecprio = (int)(n*0.45);
			int sinsert = (int)(n*0.08);
			int sdelete = (int)(n*0.01);
			int sdelmin = (int)(n*0.01);
			System.out.println("Elemente: " + n);
			for(int i=1; i<=runs; i++){
				System.out.print(i+",");
				Integer[] keys = new Integer[sinit];
				int[] prios = new int[sinit];
				for(int j=0; j<keys.length; j++){
					keys[j] = j;
					prios[j] = rnd.nextInt(n*10) + 1;
				}
				int[] newPrios = new int[sdecprio];
				for(int j=0; j<newPrios.length; j++)
					newPrios[j] = (int)(prios[j]*(rnd.nextDouble()+0.1)*0.8);
				Integer[] insertKeys = new Integer[sinsert];
				int[] insertPrios = new int[sinsert];
				for(int j=0; j<insertKeys.length; j++){
					insertKeys[j] = keys.length + j;
					insertPrios[j] = rnd.nextInt(n*10) + 1;
				}
				
				long ts = timer.getCurrentThreadCpuTime();
				// init
				LeftistTree<Integer,Integer> ltree = new LeftistTree<Integer,Integer>(keys,keys,prios);
				// delete
				for(int j=0; j<sdelete; j++){
					ltree.delete(ltree.getNode(j));
				}
				// insert
				for(int j=0; j<insertKeys.length; j++){
					ltree.insert(new LeftistTreeNode<Integer,Integer>(insertKeys[j],insertKeys[j],insertPrios[j]));
				}
				// decrease priority
				for(int j=0; j<sdecprio; j++){
					ltree.decreasePriority(ltree.getNode(j), newPrios[j]);
				}
				// delete minimum
				for(int j=0; j<sdelmin; j++){
					ltree.deleteMinimum();
				}
				long te = timer.getCurrentThreadCpuTime();
				scenTime[0][k] += (te-ts)*Math.pow(10,-9);
				
				ts = timer.getCurrentThreadCpuTime();
				// init
				BinomialHeap<Integer,Integer> binheap = new BinomialHeap<Integer,Integer>(keys,keys,prios);
				// delete
				for(int j=0; j<sdelete; j++){
					binheap.delete(binheap.getNode(j));
				}
				// insert
				for(int j=0; j<insertKeys.length; j++){
					binheap.insert(new BinomialHeapNode<Integer,Integer>(insertKeys[j],insertKeys[j],insertPrios[j]));
				}
				// decrease priority
				for(int j=0; j<sdecprio; j++){
					binheap.decreasePriority(binheap.getNode(j), newPrios[j]);
				}
				// delete minimum
				for(int j=0; j<sdelmin; j++){
					binheap.deleteMinimum();
				}
				te = timer.getCurrentThreadCpuTime();
				scenTime[1][k] += (te-ts)*Math.pow(10,-9);
			}
			System.out.println();
			scenTime[0][k] /= runs;
			scenTime[1][k] /= runs;
			System.out.println("LeftistTree: scenario32Time="+scenTime[0][k]+"s");
			System.out.println("BinHeap: scenario32Time="+scenTime[1][k]+"s");
		}
		
		Writer osscen32 = new FileWriter("Scenario32Cmp.txt");
		for(int i=0; i<2; i++){
			switch(i){
			case 0:
				osscen32.write("# Leftist Tree\n");
				break;
			case 1:
				osscen32.write("# Binomial Heap\n");
				break;
			}
			for(int j=0; j<scenTime[0].length; j++){
				osscen32.write((div+j*div) + " " + scenTime[i][j] + "\n");
			}
			osscen32.write("\n\n");
		}
		osscen32.close();
	}
	
	public static void main(String[] args) {
		// run all test cases
		try {
			initInsertTests();
			getMinDelMinTests();
			deleteTests();
			decreasePrioTests();
			mergeTests();
			scenario1();
			scenario2();
			scenario31();
			scenario32();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}
